/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#ifndef __STRING_H__
#define __STRING_H__

#include <pthread.h>

typedef struct StringList {
	int Entries;
	char **Entry;
	pthread_mutex_t Mutex;
} StringList;

void Init_StringList(StringList *List);
void Destroy_StringList(StringList *List);
void Remove_Trailing_Spaces(char *String);
void Remove_Trailing_Whitespaces(char *String);
void Remove_Leading_Whitespaces(char *String);
int String_Exists(const StringList *List, const char *String);
void Lock_Stringlist(StringList *List);
void Unlock_Stringlist(StringList *List);
void Add_String(StringList *List, const char *String);
void Add_String_Atomic(StringList *List, const char *String);
void Add_String_Unique(StringList *List, const char *String);
void Add_String_Unique_Atomic(StringList *List, const char *String);
void Remove_String(StringList *List, int Index);
void String_Lowercase(char *String);
const char *SingularPlural(const char *Singular, const char *Plural, int Number);
void Date_To_String(char *Buffer, int Size, time_t Time);
const char *GetSmblisterError(int Status);

#endif
