/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#ifndef __THREADS_H_
#define __THREADS_H_

#include "IP_Range.h"
#include "ProcessTable.h"
#include "Semaphore.h"
#include "Hostlist.h"

struct Process_Watcher_Thread_InputData {
	char Quit;
	struct Semaphore_Data Exit_Semaphore;
};

struct Redirect_Thread_InputData {
	int Outfile_FD;
	int Read_FD;
	struct Process *Proc;
	double TermTime;
	double KillTime;
	pthread_mutex_t Active;
	int UniqueThreadID;
};

struct Process_Host_Thread_InputData {
	char *Hostname;
	ipv4addr_t Host_IP;
	HostList *Hostlist;
	StringList *Sharelist;
	
	pthread_mutex_t *NumberMutex;			/* Protects "Failed" and "Succeeded" */
	int *Hostlookup_Processes_Failed;
	int *Hostlookup_Processes_Succeeded;
	int PreserveTemp;

	int UniqueThreadID;
};

struct Process_Share_Thread_InputData {
	char *Sharename;
	StringList *Resultfilelist;
	int UniqueThreadID;
};

struct Process_IP_Address_Thread_InputData {
	ipv4addr_t IP_Address;
	HostList *Hostlist;
	int UniqueThreadID;
	int PreserveTemp;
};

int GetUniqueThreadID();
void* Redirect_Thread(void *Parameters);
void* Process_Watcher_Thread(void *Parameters);
void* Process_Host_Thread(void *Parameters);
void* Process_Share_Thread(void *Parameters);
void* Process_IP_Address_Thread(void *Parameters);

#endif
