/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "IP_Range.h"
#include "String.h"
#include "IP_Hostname_Pairings.h"
#include "sambascanner.h"

static StringList Pairings;
static ipv4addr_t *IPs = NULL;
static pthread_mutex_t PairingLock;

void Init_Pairings() {
	pthread_mutex_init(&PairingLock, NULL);
	Init_StringList(&Pairings);
}

/* The order of operations is important here - as the increment of Pairings.Entries
 * is the very last thing happening, read access is always safe without locking.
 * We need, however, the PairingLock as Add_IP_Hostname_Pairing might interrupt itself. */
void Add_IP_Hostname_Pairing(const ipv4addr_t IP, const char *Hostname) {
	pthread_mutex_lock(&PairingLock);
	IPs = (ipv4addr_t*)realloc(IPs, sizeof(ipv4addr_t) * (Pairings.Entries + 1));
	IPs[Pairings.Entries] = IP;
	Add_String(&Pairings, Hostname);
	pthread_mutex_unlock(&PairingLock);
}

int Get_IP_From_Hostname(ipv4addr_t *IP, const char *Hostname) {
	int i;
	for (i = 0; i < Pairings.Entries; i++) {
		if (!strcasecmp(Hostname, Pairings.Entry[i])) {
			*IP = IPs[i];
			return 1;
		}
	}
	return 0;
}

void Destroy_Pairings() {
	free(IPs);
	IPs = NULL;
	Destroy_StringList(&Pairings);
	pthread_mutex_destroy(&PairingLock);
}

void Show_Pairings() {
	int i;
	for (i = 0; i < Pairings.Entries; i++) {
		fprintf(stderr, "%3d: %d.%d.%d.%d -> %s\n", i + 1, IP_ADDRESS(IPs[i]), Pairings.Entry[i]);
	}
}

