/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <libintl.h>

#include "sambascanner.h"
#include "Semaphore.h"
#include "PThreads.h"

void Semaphore_Init(struct Semaphore_Data *SemData, int InitValue, const char *Name) {
	SemData->Value = InitValue;
	Safe_pthread_mutex_init(&(SemData->Mutex), Name);
	Safe_pthread_cond_init(&(SemData->Signal), Name);
}

void Semaphore_SetValue(struct Semaphore_Data *SemData, int Value) {
	SemData->Value = Value;
}

/* Decrement semaphore */
void Semaphore_P(struct Semaphore_Data *SemData, int Value) {
	Safe_pthread_mutex_lock(&(SemData->Mutex));
	while (SemData->Value < Value) {
		pthread_cond_wait(&(SemData->Signal), &(SemData->Mutex));
	}
	SemData->Value -= Value;
	Safe_pthread_mutex_unlock(&(SemData->Mutex));
}

/* Increment semaphore */
void Semaphore_V(struct Semaphore_Data *SemData, int Value) {
	Safe_pthread_mutex_lock(&(SemData->Mutex));
	SemData->Value += Value;
	pthread_cond_broadcast(&(SemData->Signal));
	Safe_pthread_mutex_unlock(&(SemData->Mutex));
}

void Semaphore_Free(struct Semaphore_Data *SemData, const char *Name) {
	Safe_pthread_mutex_destroy(&(SemData->Mutex), Name);
	Safe_pthread_cond_destroy(&(SemData->Signal), Name);
}

